"""
Configs migration

Generation datetime: 2021-12-20 12:42:19.792322
"""

import warnings
from typing import List

import fastjsonschema
from configs_core.migration.db_migrate_helper import DatabaseMigrationHelper, LimitationToModify

mh: DatabaseMigrationHelper

REVISION = "4cfc5f13"
DOWN_REVISION = "096fc21a"


INFLUX1_SCHEMA = {
    "properties": {
        "send_data_for_monitoring": {"type": "integer", "enum": [0, 1]},
        "use_ssl": {"type": "integer", "enum": [0, 1]},
        "flushing_period": {"type": "number"},
        "host": {"type": "string"},
        "port": {"type": "integer", "minimum": 1, "maximum": 65535},
        "database_name": {"type": "string"},
        "user_name": {"type": "string"},
        "password": {"type": "string"},
        "version": {"type": "integer", "enum": [1]},
    },
    "required": ["send_data_for_monitoring", "host", "port", "database_name"],
    "type": "object",
    "additionalProperties": False,
}


def upgradeInfluxMonitoring(item: dict) -> dict:
    try:
        fastjsonschema.validate(INFLUX1_SCHEMA, item)
    except fastjsonschema.exceptions.JsonSchemaException:
        item.pop("version", 2)
        return item

    if item.get("send_data_for_monitoring", 0) == 1:
        warnings.warn("Influx 1.x is not supported anymore", DeprecationWarning)

    return {
        "send_data_for_monitoring": 0,
        "use_ssl": 0,
        "flushing_period": 1,
        "host": "127.0.0.1",
        "port": 8086,
        "bucket": "luna_monitoring",
        "organization": "luna",
        "token": "secret-token",
    }


def downgradeInfluxMonitoring(item: dict) -> dict:
    item["version"] = 2
    return item


limitationsToModify: List[LimitationToModify] = [
    LimitationToModify(
        limitationName="INFLUX_MONITORING",
        oldValidationSchema={
            "type": "object",
            "description": "Monitoring settings.",
            "oneOf": [
                {
                    "properties": {
                        "send_data_for_monitoring": {"type": "integer", "enum": [0, 1]},
                        "use_ssl": {"type": "integer", "enum": [0, 1]},
                        "flushing_period": {"type": "number"},
                        "host": {"type": "string"},
                        "port": {"type": "integer", "minimum": 1, "maximum": 65535},
                        "database_name": {"type": "string"},
                        "user_name": {"type": "string"},
                        "password": {"type": "string"},
                        "version": {"type": "integer", "enum": [1]},
                    },
                    "default": {
                        "version": 1,
                        "use_ssl": 0,
                        "flushing_period": 1,
                        "user_name": "luna",
                        "password": "luna",
                    },
                    "required": [
                        "send_data_for_monitoring",
                        "host",
                        "port",
                        "database_name",
                    ],
                    "type": "object",
                    "additionalProperties": False,
                },
                {
                    "properties": {
                        "send_data_for_monitoring": {"type": "integer", "enum": [0, 1]},
                        "use_ssl": {"type": "integer", "enum": [0, 1]},
                        "flushing_period": {"type": "number"},
                        "host": {"type": "string"},
                        "port": {"type": "integer", "minimum": 1, "maximum": 65535},
                        "organization": {"type": "string"},
                        "token": {"type": "string"},
                        "bucket": {"type": "string"},
                        "version": {"type": "integer", "enum": [2]},
                    },
                    "default": {"version": 2, "use_ssl": 0, "flushing_period": 1},
                    "required": [
                        "send_data_for_monitoring",
                        "host",
                        "port",
                        "organization",
                        "token",
                        "bucket",
                    ],
                    "type": "object",
                    "additionalProperties": False,
                },
            ],
        },
        newValidationSchema={
            "properties": {
                "send_data_for_monitoring": {"type": "integer", "enum": [0, 1]},
                "use_ssl": {"type": "integer", "enum": [0, 1]},
                "flushing_period": {"type": "number"},
                "host": {"type": "string"},
                "port": {"type": "integer", "minimum": 1, "maximum": 65535},
                "organization": {"type": "string"},
                "token": {"type": "string"},
                "bucket": {"type": "string"},
            },
            "default": {
                "send_data_for_monitoring": 1,
                "use_ssl": 0,
                "flushing_period": 1,
                "host": "127.0.0.1",
                "port": 8086,
                "organization": "luna",
                "token": "secret-token",
                "bucket": "luna_monitoring",
            },
            "required": [
                "send_data_for_monitoring",
                "host",
                "port",
                "organization",
                "token",
                "bucket",
            ],
            "type": "object",
            "additionalProperties": False,
            "description": "Monitoring settings.",
        },
        oldDefaultValue={
            "send_data_for_monitoring": 1,
            "version": 1,
            "use_ssl": 0,
            "flushing_period": 1,
            "host": "127.0.0.1",
            "port": 8086,
            "database_name": "luna_monitoring",
            "user_name": "luna",
            "password": "luna",
        },
        newDefaultValue={
            "send_data_for_monitoring": 1,
            "use_ssl": 0,
            "flushing_period": 1,
            "host": "127.0.0.1",
            "port": 8086,
            "bucket": "luna_monitoring",
            "organization": "luna",
            "token": "secret-token",
        },
        settingUpgradeFunction=upgradeInfluxMonitoring,
        settingDowngradeFunction=downgradeInfluxMonitoring,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=[
            "luna-admin",
            "luna-api",
            "luna-backport3",
            "luna-backport4",
            "luna-configurator",
            "luna-events",
            "luna-faces",
            "luna-handlers",
            "luna-image-store",
            "luna-index-manager",
            "luna-licenses",
            "luna-matcher-proxy",
            "luna-python-matcher",
            "luna-sender",
            "luna-tasks",
        ],
        newServices=[
            "luna-admin",
            "luna-api",
            "luna-backport3",
            "luna-backport4",
            "luna-configurator",
            "luna-events",
            "luna-faces",
            "luna-handlers",
            "luna-image-store",
            "luna-index-manager",
            "luna-licenses",
            "luna-matcher-proxy",
            "luna-python-matcher",
            "luna-sender",
            "luna-tasks",
        ],
    ),
]


def upgrade():
    """Upgrade configs to new revision"""

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)


def downgrade():
    """Downgrade configs to previous revision"""

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)
