"""
Configs migration

Generation datetime: 2022-11-03 10:44:09.629210
"""

from typing import List, NamedTuple

import ujson
from configs_core.migration import models
from configs_core.migration.db_migrate_helper import (
    DatabaseMigrationHelper,
    ExternalLimitationToModify,
    LimitationToModify,
)
from sqlalchemy import insert, outerjoin, select, update

mh: DatabaseMigrationHelper

REVISION = "1345bff2"
DOWN_REVISION = "acbae50c"

estimatorsDescriptions = {
    "LUNA_HANDLERS_BASIC_ATTRIBUTES_ESTIMATOR_SETTINGS": "luna-handlers settings for basic attributes estimation in handlers",
    "LUNA_HANDLERS_BODY_ATTRIBUTES_ESTIMATOR_SETTINGS": "luna-handlers settings for body attributes estimation in handlers",
    "LUNA_HANDLERS_BODY_DESCRIPTOR_ESTIMATOR_SETTINGS": "luna-handlers settings for body descriptor estimation in handlers",
    "LUNA_HANDLERS_BODY_LANDMARKS_ESTIMATOR_SETTINGS": "luna-handlers settings for body landmarks estimation in handlers",
    "LUNA_HANDLERS_BODY_WARP_ESTIMATOR_SETTINGS": "luna-handlers settings for body warp estimation in handlers",
    "LUNA_HANDLERS_EMOTIONS_ESTIMATOR_SETTINGS": "luna-handlers settings for emotions estimation in handlers",
    "LUNA_HANDLERS_EYEBROW_EXPRESSION_ESTIMATOR_SETTINGS": "luna-handlers settings for eyebrow expression estimation in handlers",
    "LUNA_HANDLERS_EYES_ESTIMATOR_SETTINGS": "luna-handlers settings for eyes estimation in handlers",
    "LUNA_HANDLERS_FACE_DESCRIPTOR_ESTIMATOR_SETTINGS": "luna-handlers settings for face descriptor estimation in handlers",
    "LUNA_HANDLERS_FACE_DETECTION_BACKGROUND_ESTIMATOR_SETTINGS": "luna-handlers settings for face detection background estimation in handlers",
    "LUNA_HANDLERS_FACE_LANDMARKS5_ESTIMATOR_SETTINGS": "luna-handlers settings for face landmarks5 estimation in handlers",
    "LUNA_HANDLERS_FACE_LANDMARKS68_ESTIMATOR_SETTINGS": "luna-handlers settings for face landmarks68 estimation in handlers",
    "LUNA_HANDLERS_FACE_NATURAL_LIGHT_ESTIMATOR_SETTINGS": "luna-handlers settings for face natural light estimation in handlers",
    "LUNA_HANDLERS_FACE_WARP_ESTIMATOR_SETTINGS": "luna-handlers settings for face warp estimation in handlers",
    "LUNA_HANDLERS_FISHEYE_ESTIMATOR_SETTINGS": "luna-handlers settings for fisheye estimation in handlers",
    "LUNA_HANDLERS_GAZE_ESTIMATOR_SETTINGS": "luna-handlers settings for gaze estimation in handlers",
    "LUNA_HANDLERS_GLASSES_ESTIMATOR_SETTINGS": "luna-handlers settings for glasses estimation in handlers",
    "LUNA_HANDLERS_HEADWEAR_ESTIMATOR_SETTINGS": "luna-handlers settings for headwear estimation in handlers",
    "LUNA_HANDLERS_HEAD_POSE_ESTIMATOR_SETTINGS": "luna-handlers settings for head pose estimation in handlers",
    "LUNA_HANDLERS_HUMAN_DETECTOR_SETTINGS": "luna-handlers settings for human detection in handlers",
    "LUNA_HANDLERS_IMAGE_COLOR_TYPE_ESTIMATOR_SETTINGS": "luna-handlers settings for image color type estimation in handlers",
    "LUNA_HANDLERS_IMAGE_ORIENTATION_ESTIMATOR_SETTINGS": "luna-handlers settings for image orientation estimation in handlers",
    "LUNA_HANDLERS_MASK_ESTIMATOR_SETTINGS": "luna-handlers settings for mask estimation in handlers",
    "LUNA_HANDLERS_MOUTH_ATTRIBUTES_ESTIMATOR_SETTINGS": "luna-handlers settings for mouth attributes estimation in handlers",
    "LUNA_HANDLERS_QUALITY_ESTIMATOR_SETTINGS": "luna-handlers settings for quality estimation in handlers",
    "LUNA_HANDLERS_RED_EYES_ESTIMATOR_SETTINGS": "luna-handlers settings for red eyes estimation in handlers",
}

otherSettingsDescriptions = {
    "LUNA_HANDLERS_BODY_DETECTOR_SETTINGS": "luna-handlers settings for body detection in handlers",
    "LUNA_HANDLERS_FACE_DETECTOR_SETTINGS": "luna-handlers settings for face detector",
    "LUNA_HANDLERS_LIVENESS_ESTIMATOR_SETTINGS": "luna-handlers settings for liveness estimation in handlers",
    "LUNA_HANDLERS_RUNTIME_SETTINGS": "runtime settings for luna-handlers",
}

allDescriptions = estimatorsDescriptions | otherSettingsDescriptions

limitationsToModify: List[LimitationToModify] = [
    LimitationToModify(
        limitationName="LUNA_HANDLERS_USE_AUTO_ROTATION",
        oldValidationSchema={"type": "boolean"},
        newValidationSchema={"type": "boolean"},
        oldDefaultValue=False,
        newDefaultValue=False,
        oldDescription="whether to try estimate image rotation angle and rotate to normal state",
        newDescription="whether to try estimate image rotation angle and rotate to normal state",
        oldGroupNames=["luna-platform", "luna-api sdk settings"],
        newGroupNames=["luna-platform", "luna-handlers-estimators-settings"],
        settingUpgradeFunction=None,
        settingDowngradeFunction=None,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-handlers"],
        newServices=["luna-handlers"],
    ),
    LimitationToModify(
        limitationName="LUNA_HANDLERS_DETECTOR_TYPE",
        oldValidationSchema={"type": "string", "enum": ["FACE_DET_V3"]},
        newValidationSchema={"type": "string", "enum": ["FACE_DET_V3"]},
        oldDefaultValue="FACE_DET_V3",
        newDefaultValue="FACE_DET_V3",
        oldDescription="luna-handlers face detector type",
        newDescription="luna-handlers face detector type",
        oldGroupNames=["luna-platform", "luna-api sdk settings"],
        newGroupNames=["luna-platform", "luna-handlers-estimators-settings"],
        settingUpgradeFunction=None,
        settingDowngradeFunction=None,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-handlers"],
        newServices=["luna-handlers"],
    ),
]

externalLimitationsToModify: List[ExternalLimitationToModify] = []


class SettingsRelations(NamedTuple):
    """Structure that discribes relations between old and new settings"""

    oldRuntimeSettings: dict
    oldSettings: dict
    newSettingsList: List[str]
    estimSettings: dict[str, dict] = {}


def getSetting(name: str) -> dict:
    """
    Get setting by name from db.
    Args:
        name: luna-handlers setting name
    Returns:
        settings as dict
    """
    record = mh.connection.execute(
        select(
            [models.Tag.tag, models.Setting.value],
            from_obj=outerjoin(models.Setting, models.Tag, models.Setting.id == models.Tag.id),
        ).where(models.Setting.name == name)
    ).cursor.fetchall()
    settings = {tag: ujson.loads(value) for tag, value in dict(record).items()}
    return settings


def setSetting(value: dict, name: str, tag: str = None):
    """
    Set setting by name.
    Args:
        value: settings to be set
        name: luna-handlers setting name
    """
    if tag is None:
        mh.connection.execute(
            update(models.Setting)
            .where(models.Setting.name == name and models.Setting.is_default == True)
            .values(value=ujson.dumps(value, indent=1))
        )
    else:
        settingQuery = insert(models.Setting).values(  # type: ignore
            value=ujson.dumps(value, indent=1), description=allDescriptions[name], name=name, is_default=False
        )
        record = mh.connection.execute(settingQuery)
        tagQuery = insert(models.Tag).values(id=record.inserted_primary_key[0], name=name, tag=tag)
        mh.connection.execute(tagQuery)


def migrateEstimatorsSettings(relation: SettingsRelations):
    """
    Migrate settings as settings relation describes.
    Args:
        relation: relation between old and new settings
    """
    estimatorsSetting = {}
    if relation.estimSettings:
        for newSetting, oldSetting in relation.estimSettings.items():
            estimatorsSetting[newSetting] = {}
            for tag, setting in oldSetting.items():
                estimatorsSetting[newSetting][tag] = {"estimator_settings": setting}

    tags = relation.oldRuntimeSettings.keys() | relation.oldSettings.keys()

    for newSetting in relation.newSettingsList:
        for tag in tags:
            oldRuntimeSetting = relation.oldRuntimeSettings.get(tag, relation.oldRuntimeSettings[None])
            oldSetting = relation.oldSettings.get(tag, relation.oldSettings[None])
            value = {
                "runtime_settings": {
                    "device_class": oldRuntimeSetting["device_class"],
                    "optimal_batch_size": oldSetting["optimal_batch_size"],
                    "worker_count": oldSetting["worker_count"],
                }
            }
            if newSetting in estimatorsSetting:
                for estimTag, estimSetting in estimatorsSetting[newSetting].items():
                    setSetting(value | estimSetting, newSetting, estimTag)
            else:
                setSetting(value, newSetting, tag)


def upgrade():
    """Upgrade configs to new revision"""

    relations = [
        SettingsRelations(
            getSetting("LUNA_HANDLERS_DETECTOR_RUNTIME_SETTINGS"),
            getSetting("LUNA_HANDLERS_DETECT_FACES_STAGE_SETTINGS"),
            [
                "LUNA_HANDLERS_FACE_DETECTOR_SETTINGS",
                "LUNA_HANDLERS_IMAGE_ORIENTATION_ESTIMATOR_SETTINGS",
                "LUNA_HANDLERS_FACE_WARP_ESTIMATOR_SETTINGS",
                "LUNA_HANDLERS_HEAD_POSE_ESTIMATOR_SETTINGS",
                "LUNA_HANDLERS_EYES_ESTIMATOR_SETTINGS",
                "LUNA_HANDLERS_GAZE_ESTIMATOR_SETTINGS",
                "LUNA_HANDLERS_LIVENESS_ESTIMATOR_SETTINGS",
            ],
            {
                "LUNA_HANDLERS_FACE_DETECTOR_SETTINGS": getSetting("FACE_DETECTOR_V3"),
                "LUNA_HANDLERS_LIVENESS_ESTIMATOR_SETTINGS": getSetting("LUNA_HANDLERS_LIVENESS_SETTINGS"),
            },
        ),
        SettingsRelations(
            getSetting("LUNA_HANDLERS_WARP_ESTIMATOR_RUNTIME_SETTINGS"),
            getSetting("LUNA_HANDLERS_WARP_ESTIMATE_STAGE_SETTINGS"),
            [
                "LUNA_HANDLERS_FACE_WARP_ESTIMATOR_SETTINGS",
                "LUNA_HANDLERS_EMOTIONS_ESTIMATOR_SETTINGS",
                "LUNA_HANDLERS_MOUTH_ATTRIBUTES_ESTIMATOR_SETTINGS",
                "LUNA_HANDLERS_MASK_ESTIMATOR_SETTINGS",
                "LUNA_HANDLERS_GLASSES_ESTIMATOR_SETTINGS",
            ],
        ),
        SettingsRelations(
            getSetting("LUNA_HANDLERS_EXTRACTOR_RUNTIME_SETTINGS"),
            getSetting("LUNA_HANDLERS_EXTRACT_STAGE_SETTINGS"),
            [
                "LUNA_HANDLERS_FACE_DESCRIPTOR_ESTIMATOR_SETTINGS",
                "LUNA_HANDLERS_BASIC_ATTRIBUTES_ESTIMATOR_SETTINGS",
            ],
        ),
        SettingsRelations(
            getSetting("LUNA_HANDLERS_HUMAN_DETECTOR_RUNTIME_SETTINGS"),
            getSetting("LUNA_HANDLERS_DETECT_HUMANS_STAGE_SETTINGS"),
            [
                "LUNA_HANDLERS_BODY_DETECTOR_SETTINGS",
                "LUNA_HANDLERS_BODY_WARP_ESTIMATOR_SETTINGS",
                "LUNA_HANDLERS_HUMAN_DETECTOR_SETTINGS",
            ],
            {"LUNA_HANDLERS_BODY_DETECTOR_SETTINGS": getSetting("BODY_DETECTOR")},
        ),
        SettingsRelations(
            getSetting("LUNA_HANDLERS_HUMAN_EXTRACTOR_RUNTIME_SETTINGS"),
            getSetting("LUNA_HANDLERS_HUMAN_EXTRACT_STAGE_SETTINGS"),
            [
                "LUNA_HANDLERS_BODY_DESCRIPTOR_ESTIMATOR_SETTINGS",
                "LUNA_HANDLERS_BODY_ATTRIBUTES_ESTIMATOR_SETTINGS",
            ],
        ),
    ]
    # groups to remove
    mh.removeGroup(groupName="luna-api sdk settings")

    # groups to add
    mh.addGroup(groupName="luna-handlers-estimators-settings", groupDescription="")

    # limitations to remove
    mh.removeLimitation(limitationName="BODY_DETECTOR")
    mh.removeLimitation(limitationName="FACE_DETECTOR_V3")
    mh.removeLimitation(limitationName="LUNA_HANDLERS_DETECTOR_RUNTIME_SETTINGS")
    mh.removeLimitation(limitationName="LUNA_HANDLERS_DETECT_FACES_STAGE_SETTINGS")
    mh.removeLimitation(limitationName="LUNA_HANDLERS_DETECT_HUMANS_STAGE_SETTINGS")
    mh.removeLimitation(limitationName="LUNA_HANDLERS_EXTRACTOR_RUNTIME_SETTINGS")
    mh.removeLimitation(limitationName="LUNA_HANDLERS_EXTRACT_STAGE_SETTINGS")
    mh.removeLimitation(limitationName="LUNA_HANDLERS_HUMAN_DETECTOR_RUNTIME_SETTINGS")
    mh.removeLimitation(limitationName="LUNA_HANDLERS_HUMAN_EXTRACTOR_RUNTIME_SETTINGS")
    mh.removeLimitation(limitationName="LUNA_HANDLERS_HUMAN_EXTRACT_STAGE_SETTINGS")
    mh.removeLimitation(limitationName="LUNA_HANDLERS_LIVENESS_SETTINGS")
    mh.removeLimitation(limitationName="LUNA_HANDLERS_WARP_ESTIMATE_STAGE_SETTINGS")
    mh.removeLimitation(limitationName="LUNA_HANDLERS_WARP_ESTIMATOR_RUNTIME_SETTINGS")

    # limitations to add

    for name, description in estimatorsDescriptions.items():
        mh.addLimitation(
            limitationName=name,
            validationSchema={
                "type": "object",
                "properties": {
                    "runtime_settings": {
                        "type": "object",
                        "properties": {
                            "device_class": {
                                "type": "string",
                                "enum": ["global", "cpu", "gpu"],
                            },
                            "optimal_batch_size": {"type": "integer", "minimum": 1},
                            "worker_count": {"type": "integer", "minimum": 1},
                        },
                    },
                    "additionalProperties": False,
                },
                "description": "settings for sdk estimator",
                "additionalProperties": False,
            },
            defaultValue={
                "runtime_settings": {
                    "device_class": "global",
                    "optimal_batch_size": 10,
                    "worker_count": 1,
                }
            },
            description=description,
            services=["luna-handlers"],
            groupNames=["luna-platform", "luna-handlers-estimators-settings"],
        )

    mh.addLimitation(
        limitationName="LUNA_HANDLERS_BODY_DETECTOR_SETTINGS",
        validationSchema={
            "type": "object",
            "properties": {
                "runtime_settings": {
                    "type": "object",
                    "properties": {
                        "device_class": {
                            "type": "string",
                            "enum": ["global", "cpu", "gpu"],
                        },
                        "optimal_batch_size": {"type": "integer", "minimum": 1},
                        "worker_count": {"type": "integer", "minimum": 1},
                    },
                    "additionalProperties": False,
                },
                "estimator_settings": {
                    "type": "object",
                    "properties": {
                        "image_size": {"type": "integer", "minimum": 1},
                        "redetect_score_threshold": {"type": "number"},
                        "score_threshold": {"type": "number"},
                        "landmarks17_threshold": {"type": "number"},
                    },
                    "description": "settings for human body detector",
                    "additionalProperties": False,
                },
            },
            "description": "settings for body detector",
            "additionalProperties": False,
        },
        defaultValue={
            "runtime_settings": {
                "device_class": "global",
                "optimal_batch_size": 10,
                "worker_count": 1,
            },
            "estimator_settings": {
                "image_size": 640,
                "redetect_score_threshold": 0.12,
                "score_threshold": 0.3,
                "landmarks17_threshold": 0.25,
            },
        },
        description="luna-handlers settings for body detection in handlers",
        services=["luna-handlers"],
        groupNames=["luna-platform", "luna-handlers-estimators-settings"],
    )
    mh.addLimitation(
        limitationName="LUNA_HANDLERS_FACE_DETECTOR_SETTINGS",
        validationSchema={
            "type": "object",
            "properties": {
                "runtime_settings": {
                    "type": "object",
                    "properties": {
                        "device_class": {
                            "type": "string",
                            "enum": ["global", "cpu", "gpu"],
                        },
                        "optimal_batch_size": {"type": "integer", "minimum": 1},
                        "worker_count": {"type": "integer", "minimum": 1},
                    },
                    "additionalProperties": False,
                },
                "estimator_settings": {
                    "type": "object",
                    "properties": {
                        "min_face_size": {"type": "integer", "minimum": 1},
                        "max_face_size": {"type": "integer", "minimum": 1},
                        "redetect_tensor_size": {"type": "integer", "minimum": 1},
                        "redetect_face_target_size": {"type": "integer", "minimum": 1},
                        "redetect_score_threshold": {"type": "number"},
                        "score_threshold": {"type": "number"},
                    },
                    "description": "settings for face detector version 3",
                    "additionalProperties": False,
                },
            },
            "description": "settings for sdk face detector",
            "additionalProperties": False,
        },
        defaultValue={
            "runtime_settings": {
                "device_class": "global",
                "optimal_batch_size": 10,
                "worker_count": 1,
            },
            "estimator_settings": {
                "max_face_size": 640,
                "min_face_size": 50,
                "redetect_tensor_size": 80,
                "redetect_face_target_size": 45,
                "redetect_score_threshold": 0.3,
                "score_threshold": 0.42,
            },
        },
        description="luna-handlers settings for face detector",
        services=["luna-handlers"],
        groupNames=["luna-platform", "luna-handlers-estimators-settings"],
    )
    mh.addLimitation(
        limitationName="LUNA_HANDLERS_LIVENESS_ESTIMATOR_SETTINGS",
        validationSchema={
            "type": "object",
            "properties": {
                "runtime_settings": {
                    "type": "object",
                    "properties": {
                        "device_class": {
                            "type": "string",
                            "enum": ["global", "cpu", "gpu"],
                        },
                        "optimal_batch_size": {"type": "integer", "minimum": 1},
                        "worker_count": {"type": "integer", "minimum": 1},
                    },
                    "additionalProperties": False,
                },
                "estimator_settings": {
                    "type": "object",
                    "properties": {
                        "real_threshold": {
                            "type": "number",
                            "minimum": 0,
                            "maximum": 1,
                        },
                        "quality_threshold": {
                            "type": "number",
                            "minimum": 0,
                            "maximum": 1,
                        },
                    },
                    "description": "FSDK one shot liveness settings",
                    "additionalProperties": False,
                },
            },
            "description": "settings for sdk liveness estimator",
            "additionalProperties": False,
        },
        defaultValue={
            "runtime_settings": {
                "device_class": "global",
                "optimal_batch_size": 10,
                "worker_count": 1,
            },
            "estimator_settings": {"real_threshold": 0.5, "quality_threshold": 0.5},
        },
        description="luna-handlers settings for liveness estimation in handlers",
        services=["luna-handlers"],
        groupNames=["luna-platform", "luna-handlers-estimators-settings"],
    )
    mh.addLimitation(
        limitationName="LUNA_HANDLERS_RUNTIME_SETTINGS",
        validationSchema={
            "type": "object",
            "properties": {
                "global_device_class": {"type": "string", "enum": ["cpu", "gpu"]},
                "num_threads": {"type": "integer", "minimum": 1},
                "num_compute_streams": {"type": "integer", "minimum": 1},
            },
            "description": "sdk runtime settings",
            "additionalProperties": False,
        },
        defaultValue={
            "global_device_class": "cpu",
            "num_threads": 6,
            "num_compute_streams": 4,
        },
        description="runtime settings for luna-handlers",
        services=["luna-handlers"],
        groupNames=["luna-platform", "luna-handlers-estimators-settings"],
    )

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)

    for limitation in externalLimitationsToModify:
        mh.migrateLimitation(limitation)

    for relation in relations:
        migrateEstimatorsSettings(relation)


def downgrade():
    """Downgrade configs to previous revision"""

    # limitations to modify
    for limitation in externalLimitationsToModify:
        mh.migrateLimitation(limitation)

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)

    # groups to add
    mh.addGroup(groupName="luna-api sdk settings", groupDescription="")

    # limitations to add
    mh.addLimitation(
        limitationName="BODY_DETECTOR",
        validationSchema={
            "type": "object",
            "properties": {
                "image_size": {"type": "integer", "minimum": 1},
                "redetect_score_threshold": {"type": "number"},
                "score_threshold": {"type": "number"},
                "landmarks17_threshold": {"type": "number"},
            },
            "description": "settings for human body detector",
            "additionalProperties": False,
        },
        defaultValue={
            "image_size": 640,
            "redetect_score_threshold": 0.12,
            "score_threshold": 0.3,
            "landmarks17_threshold": 0.25,
        },
        description="body detector settings",
        services=["luna-handlers"],
        groupNames=["luna-platform", "luna-api sdk settings"],
    )
    mh.addLimitation(
        limitationName="FACE_DETECTOR_V3",
        validationSchema={
            "type": "object",
            "properties": {
                "min_face_size": {"type": "integer", "minimum": 1},
                "max_face_size": {"type": "integer", "minimum": 1},
                "redetect_tensor_size": {"type": "integer", "minimum": 1},
                "redetect_face_target_size": {"type": "integer", "minimum": 1},
                "redetect_score_threshold": {"type": "number"},
                "score_threshold": {"type": "number"},
            },
            "description": "settings for face detector version 3",
            "additionalProperties": False,
        },
        defaultValue={
            "max_face_size": 640,
            "min_face_size": 50,
            "redetect_tensor_size": 80,
            "redetect_face_target_size": 45,
            "redetect_score_threshold": 0.3,
            "score_threshold": 0.42,
        },
        description="face detector version 3 settings",
        services=["luna-handlers"],
        groupNames=["luna-platform", "luna-api sdk settings"],
    )
    mh.addLimitation(
        limitationName="LUNA_HANDLERS_DETECTOR_RUNTIME_SETTINGS",
        validationSchema={
            "type": "object",
            "properties": {
                "device_class": {"type": "string", "enum": ["cpu", "gpu"]},
                "num_threads": {"type": "integer", "minimum": 1},
                "num_compute_streams": {"type": "integer", "minimum": 1},
            },
            "description": "sdk runtime settings",
            "additionalProperties": False,
        },
        defaultValue={
            "device_class": "cpu",
            "num_threads": 6,
            "num_compute_streams": 4,
        },
        description="runtime settings for face detector in handlers",
        services=["luna-handlers"],
        groupNames=["luna-platform", "luna-api sdk settings"],
    )
    mh.addLimitation(
        limitationName="LUNA_HANDLERS_DETECT_FACES_STAGE_SETTINGS",
        validationSchema={
            "type": "object",
            "properties": {
                "optimal_batch_size": {"type": "integer", "minimum": 1},
                "worker_count": {"type": "integer", "minimum": 1},
            },
            "description": "settings for handlers stage",
            "additionalProperties": False,
        },
        defaultValue={"optimal_batch_size": 8, "worker_count": 1},
        description="luna-handlers detect faces stage settings",
        services=["luna-handlers"],
        groupNames=["luna-platform", "luna-api sdk settings"],
    )
    mh.addLimitation(
        limitationName="LUNA_HANDLERS_DETECT_HUMANS_STAGE_SETTINGS",
        validationSchema={
            "type": "object",
            "properties": {
                "optimal_batch_size": {"type": "integer", "minimum": 1},
                "worker_count": {"type": "integer", "minimum": 1},
            },
            "description": "settings for handlers stage",
            "additionalProperties": False,
        },
        defaultValue={"optimal_batch_size": 8, "worker_count": 1},
        description="luna-handlers detect humans stage settings",
        services=["luna-handlers"],
        groupNames=["luna-platform", "luna-api sdk settings"],
    )
    mh.addLimitation(
        limitationName="LUNA_HANDLERS_EXTRACTOR_RUNTIME_SETTINGS",
        validationSchema={
            "type": "object",
            "properties": {
                "device_class": {"type": "string", "enum": ["cpu", "gpu"]},
                "num_threads": {"type": "integer", "minimum": 1},
                "num_compute_streams": {"type": "integer", "minimum": 1},
            },
            "description": "sdk runtime settings",
            "additionalProperties": False,
        },
        defaultValue={
            "device_class": "cpu",
            "num_threads": 6,
            "num_compute_streams": 4,
        },
        description="runtime settings for face extractor in handlers",
        services=["luna-handlers"],
        groupNames=["luna-platform", "luna-api sdk settings"],
    )
    mh.addLimitation(
        limitationName="LUNA_HANDLERS_EXTRACT_STAGE_SETTINGS",
        validationSchema={
            "type": "object",
            "properties": {
                "optimal_batch_size": {"type": "integer", "minimum": 1},
                "worker_count": {"type": "integer", "minimum": 1},
            },
            "description": "settings for handlers stage",
            "additionalProperties": False,
        },
        defaultValue={"optimal_batch_size": 13, "worker_count": 1},
        description="luna-handlers face extract stage settings",
        services=["luna-handlers"],
        groupNames=["luna-platform", "luna-api sdk settings"],
    )
    mh.addLimitation(
        limitationName="LUNA_HANDLERS_HUMAN_DETECTOR_RUNTIME_SETTINGS",
        validationSchema={
            "type": "object",
            "properties": {
                "device_class": {"type": "string", "enum": ["cpu", "gpu"]},
                "num_threads": {"type": "integer", "minimum": 1},
                "num_compute_streams": {"type": "integer", "minimum": 1},
            },
            "description": "sdk runtime settings",
            "additionalProperties": False,
        },
        defaultValue={
            "device_class": "cpu",
            "num_threads": 6,
            "num_compute_streams": 4,
        },
        description="runtime settings for human detector in handlers",
        services=["luna-handlers"],
        groupNames=["luna-platform", "luna-api sdk settings"],
    )
    mh.addLimitation(
        limitationName="LUNA_HANDLERS_HUMAN_EXTRACTOR_RUNTIME_SETTINGS",
        validationSchema={
            "type": "object",
            "properties": {
                "device_class": {"type": "string", "enum": ["cpu", "gpu"]},
                "num_threads": {"type": "integer", "minimum": 1},
                "num_compute_streams": {"type": "integer", "minimum": 1},
            },
            "description": "sdk runtime settings",
            "additionalProperties": False,
        },
        defaultValue={
            "device_class": "cpu",
            "num_threads": 6,
            "num_compute_streams": 4,
        },
        description="runtime settings for human extractor in handlers",
        services=["luna-handlers"],
        groupNames=["luna-platform", "luna-api sdk settings"],
    )
    mh.addLimitation(
        limitationName="LUNA_HANDLERS_HUMAN_EXTRACT_STAGE_SETTINGS",
        validationSchema={
            "type": "object",
            "properties": {
                "optimal_batch_size": {"type": "integer", "minimum": 1},
                "worker_count": {"type": "integer", "minimum": 1},
            },
            "description": "settings for handlers stage",
            "additionalProperties": False,
        },
        defaultValue={"optimal_batch_size": 13, "worker_count": 1},
        description="luna-handlers human extract stage settings",
        services=["luna-handlers"],
        groupNames=["luna-platform", "luna-api sdk settings"],
    )
    mh.addLimitation(
        limitationName="LUNA_HANDLERS_LIVENESS_SETTINGS",
        validationSchema={
            "type": "object",
            "properties": {
                "real_threshold": {"type": "number", "minimum": 0, "maximum": 1},
                "quality_threshold": {"type": "number", "minimum": 0, "maximum": 1},
            },
            "description": "FSDK one shot liveness settings",
            "additionalProperties": False,
        },
        defaultValue={"real_threshold": 0.5, "quality_threshold": 0.5},
        description="FSDK one shot liveness settings",
        services=["luna-api", "luna-handlers"],
        groupNames=["luna-platform", "luna-api sdk settings"],
    )
    mh.addLimitation(
        limitationName="LUNA_HANDLERS_WARP_ESTIMATE_STAGE_SETTINGS",
        validationSchema={
            "type": "object",
            "properties": {
                "optimal_batch_size": {"type": "integer", "minimum": 1},
                "worker_count": {"type": "integer", "minimum": 1},
            },
            "description": "settings for handlers stage",
            "additionalProperties": False,
        },
        defaultValue={"optimal_batch_size": 1, "worker_count": 1},
        description="luna-handlers face warp estimation stage settings",
        services=["luna-handlers"],
        groupNames=["luna-platform", "luna-api sdk settings"],
    )
    mh.addLimitation(
        limitationName="LUNA_HANDLERS_WARP_ESTIMATOR_RUNTIME_SETTINGS",
        validationSchema={
            "type": "object",
            "properties": {
                "device_class": {"type": "string", "enum": ["cpu", "gpu"]},
                "num_threads": {"type": "integer", "minimum": 1},
                "num_compute_streams": {"type": "integer", "minimum": 1},
            },
            "description": "sdk runtime settings",
            "additionalProperties": False,
        },
        defaultValue={
            "device_class": "cpu",
            "num_threads": 6,
            "num_compute_streams": 4,
        },
        description="runtime settings for face warp estimator in handlers",
        services=["luna-handlers"],
        groupNames=["luna-platform", "luna-api sdk settings"],
    )

    # limitations to remove
    mh.removeLimitation(limitationName="LUNA_HANDLERS_BASIC_ATTRIBUTES_ESTIMATOR_SETTINGS")
    mh.removeLimitation(limitationName="LUNA_HANDLERS_BODY_ATTRIBUTES_ESTIMATOR_SETTINGS")
    mh.removeLimitation(limitationName="LUNA_HANDLERS_BODY_DESCRIPTOR_ESTIMATOR_SETTINGS")
    mh.removeLimitation(limitationName="LUNA_HANDLERS_BODY_DETECTOR_SETTINGS")
    mh.removeLimitation(limitationName="LUNA_HANDLERS_BODY_LANDMARKS_ESTIMATOR_SETTINGS")
    mh.removeLimitation(limitationName="LUNA_HANDLERS_BODY_WARP_ESTIMATOR_SETTINGS")
    mh.removeLimitation(limitationName="LUNA_HANDLERS_EMOTIONS_ESTIMATOR_SETTINGS")
    mh.removeLimitation(limitationName="LUNA_HANDLERS_EYEBROW_EXPRESSION_ESTIMATOR_SETTINGS")
    mh.removeLimitation(limitationName="LUNA_HANDLERS_EYES_ESTIMATOR_SETTINGS")
    mh.removeLimitation(limitationName="LUNA_HANDLERS_FACE_DESCRIPTOR_ESTIMATOR_SETTINGS")
    mh.removeLimitation(limitationName="LUNA_HANDLERS_FACE_DETECTION_BACKGROUND_ESTIMATOR_SETTINGS")
    mh.removeLimitation(limitationName="LUNA_HANDLERS_FACE_DETECTOR_SETTINGS")
    mh.removeLimitation(limitationName="LUNA_HANDLERS_FACE_LANDMARKS5_ESTIMATOR_SETTINGS")
    mh.removeLimitation(limitationName="LUNA_HANDLERS_FACE_LANDMARKS68_ESTIMATOR_SETTINGS")
    mh.removeLimitation(limitationName="LUNA_HANDLERS_FACE_NATURAL_LIGHT_ESTIMATOR_SETTINGS")
    mh.removeLimitation(limitationName="LUNA_HANDLERS_FACE_WARP_ESTIMATOR_SETTINGS")
    mh.removeLimitation(limitationName="LUNA_HANDLERS_FISHEYE_ESTIMATOR_SETTINGS")
    mh.removeLimitation(limitationName="LUNA_HANDLERS_GAZE_ESTIMATOR_SETTINGS")
    mh.removeLimitation(limitationName="LUNA_HANDLERS_GLASSES_ESTIMATOR_SETTINGS")
    mh.removeLimitation(limitationName="LUNA_HANDLERS_HEADWEAR_ESTIMATOR_SETTINGS")
    mh.removeLimitation(limitationName="LUNA_HANDLERS_HEAD_POSE_ESTIMATOR_SETTINGS")
    mh.removeLimitation(limitationName="LUNA_HANDLERS_HUMAN_DETECTOR_SETTINGS")
    mh.removeLimitation(limitationName="LUNA_HANDLERS_IMAGE_COLOR_TYPE_ESTIMATOR_SETTINGS")
    mh.removeLimitation(limitationName="LUNA_HANDLERS_IMAGE_ORIENTATION_ESTIMATOR_SETTINGS")
    mh.removeLimitation(limitationName="LUNA_HANDLERS_LIVENESS_ESTIMATOR_SETTINGS")
    mh.removeLimitation(limitationName="LUNA_HANDLERS_MASK_ESTIMATOR_SETTINGS")
    mh.removeLimitation(limitationName="LUNA_HANDLERS_MOUTH_ATTRIBUTES_ESTIMATOR_SETTINGS")
    mh.removeLimitation(limitationName="LUNA_HANDLERS_QUALITY_ESTIMATOR_SETTINGS")
    mh.removeLimitation(limitationName="LUNA_HANDLERS_RED_EYES_ESTIMATOR_SETTINGS")
    mh.removeLimitation(limitationName="LUNA_HANDLERS_RUNTIME_SETTINGS")

    # groups to remove
    mh.removeGroup(groupName="luna-handlers-estimators-settings")
